# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for the Ubuntu HTML5 Theme package """

import os
import BaseHTTPServer
import threading

HTTP_SERVER_PORT = 8383

from testtools.matchers import Contains, Equals
from autopilot.matchers import Eventually
from autopilot.testcase import AutopilotTestCase
from autopilot.input import Mouse, Touch, Pointer

# from autopilot.introspection.qt import QtIntrospectionTestMixin


class RequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
    BASE_PATH_FOR_SERVED_APPS = {'rss-reader': "%s/%s" % (os.path.dirname(os.path.realpath(__file__)), '../../../../0.1/examples/apps/rss-reader')}

    def get_served_filename(self, appname, filename):
        if len(filename) == 0 or filename == '/':
            filename = 'index.html'
        print os.path.join(self.BASE_PATH_FOR_SERVED_APPS[appname], filename)
        return os.path.join(self.BASE_PATH_FOR_SERVED_APPS[appname], filename)

    def serve_file(self, filename):
        import mimetypes
        
        content_type = mimetypes.guess_type(filename)[0]

        f = open(filename, 'rb')
        self.send_response(200)
        self.send_header('Content-length', os.fstat(f.fileno())[6])
        self.send_header('Content-type', content_type)
        self.end_headers()
        self.wfile.write(f.read())

    def do_GET(self):
        # FIXME: more generic & cleaner
        if self.path.startswith('/rss-reader'):
            filename = self.path[len('/rss-reader'):]
            self.send_response(200)
            self.serve_file(self.get_served_filename('rss-reader', filename))
        else:
            self.send_error(404)


class UbuntuHTML5HTTPServer(threading.Thread):
    def __init__(self, port):
        super(UbuntuHTML5HTTPServer, self).__init__()
        self.server = BaseHTTPServer.HTTPServer(("", port), RequestHandler)
        self.server.allow_reuse_address = True

    def run(self):
        self.server.serve_forever()

    def shutdown(self):
        self.server.shutdown()
        self.server.server_close()


class UbuntuHTML5TestCaseBase(AutopilotTestCase):
    BROWSER_CONTAINER_PATH = "%s/%s" % (os.path.dirname(os.path.realpath(__file__)), '../../tools/qml/webview.qml')
    BROWSER_QML_APP_LAUNCHER = 'qmlscene'

    BASE_URL = ''

    def setUp(self):
        self.pointer = Pointer(Mouse.create())
        self.app = self.launch_test_application(self.BROWSER_QML_APP_LAUNCHER, self.BROWSER_CONTAINER_PATH)
        super(UbuntuHTML5TestCaseBase, self).setUp()

    def tearDown(self):
        super(UbuntuHTML5TestCaseBase, self).tearDown()

    def pick_app_launcher(self, app_path):
        # force Qt app introspection:
        from autopilot.introspection.qt import QtApplicationLauncher
        return QtApplicationLauncher()

    def get_webviewContainer(self):
        return self.app.select_single(objectName="webviewContainer")

    def get_webview(self):
        return self.app.select_single(objectName="webview")

    def get_addressbar(self):
        return self.app.select_single(objectName="addressbar")

    def get_button(self):
        return self.app.select_single(objectName="browseButton")

    def get_title(self):
        return self.get_webview().title

    def assert_url_eventually_loaded(self, url):
        webview = self.get_webview()
        self.assertThat(webview.loadProgress, Eventually(Equals(100)))
        self.assertThat(webview.loading, Eventually(Equals(False)))
        self.assertThat(webview.url, Eventually(Equals(url)))

    # TODO: complete this
    def click(self, id):
        webview = self.get_webviewContainer()
        watcher = webview.watch_signal('messageReceived(QString)')
        webview.slots.clickElementById(id)
        #        self.assertThat(watcher.num_emissions, Eventually(Equals(1)))

    def browse_to_app(self, appname):
        url = self.BASE_URL + '/' + appname

        addressbar = self.get_addressbar()
        self.pointer.move_to_object(addressbar)
        self.pointer.click()

        self.keyboard.type(url, 0.001)

        button = self.get_button()
        self.pointer.move_to_object(button)
        self.pointer.click()

        self.assert_url_eventually_loaded(url);


class UbuntuThemeWithHttpServerTestCaseBase(UbuntuHTML5TestCaseBase):
    def setUp(self):
        self.server = UbuntuHTML5HTTPServer(HTTP_SERVER_PORT)
        self.server.start()
        super(UbuntuThemeWithHttpServerTestCaseBase, self).setUp()

    def tearDown(self):
        super(UbuntuThemeWithHttpServerTestCaseBase, self).tearDown()
        self.server.shutdown()


class UbuntuThemeRemotePageTestCaseBase(UbuntuThemeWithHttpServerTestCaseBase):
    def setUp(self):
        self.BASE_URL = "http://localhost:%d" % HTTP_SERVER_PORT
        super(UbuntuThemeRemotePageTestCaseBase, self).setUp()



